<?php declare (strict_types = 1);

namespace USKit\Base\ViewModels;

use USKit\Base\Parts\Image;
use Infomaker\Everyware\NewsML\Objects\TeaserItem;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;

/**
 * Class TeaserArticle
 * @package EuKit\Base
 */
class Teaser extends BaseObject
{
    public function __construct(PropertyObject $article)
    {
        $bodyraw = html_entity_decode($article->getFirstOf('body_raw'));

        if (!empty($article->getSingleValue('legacy_id'))) {
            $legacy_id = $article->getSingleValue('legacy_id');
        } else {
            $legacy_id = '';
        };

        if (!empty($article->getSingleValue('inline_images'))) {
            $inline_images = $article->getSingleValue('inline_images');
        } else {
            $inline_images = '';
        };

        $teaser = $article->getNewsMLTeaser();
        $propertyObject = new GenericPropertyObject();
        $propertyObject->fill([
            'uuid' => $article->getSingleValue('uuid'),
            'pubdate' => $article->getPubDate(),
            'updated' => $article->getUpdatedDate(),
            'headline' => $article->getSingleValue('headline'),
            'permalink' => $article->getSingleValue('permalink'),
            'subtype' => $article->getSingleValue('subtype'),
            'story_text' => html_entity_decode($article->getFirstOf('text')),
            'leadin' => $article->getFirstOf('leadin'),
            'image_uuid' => $article->getFirstOf('image_uuids'),
            'image_gallery_uuids' => $article->getMultiValue('teaser_image_uuids'),
            'search_image_uuid' => $article->getFirstOf('image_uuids'),
            'categories' => $this->getCategories($article),
            'authors' => $this->getAuthors($article),
            'bodyraw' => $bodyraw,
            'byline'  => $article->getFirstOf('byline'),
            'byline_attribution'  => $article->getFirstOf('byline_attribution'),
            'relatedArticles' => $article->articles,
            'legacy_id' => $legacy_id,
            'inline_images' => $inline_images,
            'related_sections_names' => $article->getMultiValue('related_sections_names'),
            'article_tags' => $article->getMultiValue('article_tags')
        ]);
        if ($teaser instanceof TeaserItem) {
            $propertyObject->fill($this->createTeaser($teaser));
        }

        parent::__construct($propertyObject);
    }

    private function createTeaser(TeaserItem $teaser): array
    {
        $image = $teaser->hasImage() ? new Image($teaser->getImage()) : null;

        $teaserData = array_replace($teaser->toArray(), [
            'image' => $image instanceof Image ? $image->getViewData() : null,
        ]);

        return array_filter($teaserData);
    }

    private function getCategories($article): array
    {
        $categories = [];
        $categoryArray = $article->get('categories');

        foreach ($categoryArray as $category){
            if (is_object($category)){
                $categories[] = [
                    'uuid' => $category->get('uuid'),
                    'name' => $category->get('name'),
                ];
            }
        }

        return $categories;
    }

    private function getAuthors($article): array
    {
        $authors = [];
        $authorArray = $article->get('authors');

        foreach ($authorArray as $author){
            if (is_object($author)){
                $authors[] = [
                    'uuid' => $author->get('uuid'),
                    'name' => $author->get('name'),
                    'image_uuid' => $author->get('avatar_uuid'),
                    'description' => $author->get('description'),
                ];
            }
        }

        return $authors;
    }
}
